import json

from flask_cors import CORS
from flask import Flask

app = Flask(__name__)
CORS(app)

with open('config.json') as config_file:
    config = json.load(config_file)

app.config["FILE_UPLOAD_LOCATION"] = config.get("FILE_UPLOAD_LOCATION")




from genreml.model.processing import audio


def process_audio_file(location: str) -> tuple:
    """ Given a location of an uploaded file, this will run feature extraction and return a tuple containing lists
    of features + images to be displayed to the user

    :param location - the location of a file to extract audio features from
    :returns tuple containing the lists of features extracted
    """
    audio_files = audio.AudioFiles()
    audio_files.extract_features(location, cmap=None)
    return audio_files.features, audio_files.visual_features



import os
import sys

from flask import request, render_template
from werkzeug.utils import secure_filename

from frontend import app
from frontend.processing import feature_extraction, genre_prediction


@app.route('/upload', methods=['GET', 'POST'])
def upload_file():
    """ Route for uploading audio files to the site """
    if request.method == 'POST':
        f = request.files['file']
        filename = secure_filename(f.filename)
        filepath = os.path.join(app.config['UPLOAD_FOLDER'], filename)
        f.save(filepath)
        features, visual_features = feature_extraction.process_audio_file(filepath)
        print(features)
        return render_template('index.html')


@app.route('/')
def index():
    """ Server route for the app's landing page """
    return render_template('index.html')


if __name__ == "__main__":
    environment = "production"
    if len(sys.argv) > 1:
        environment = sys.argv[1]
    if environment == "production":
        from gevent.pywsgi import WSGIServer
        app.debug = True
        http_server = WSGIServer(('', 8000), app)
        http_server.serve_forever()
    elif environment == "development":
        app.run(debug=True, host='127.0.0.1', port=8000)



from threading import Thread


def run_function_in_thread(number_of_threads: int, function_to_run, data: list, *args):
    """ Given a number of parallel threads, a function to run, some data to provide the function, and any other
    arguments the functin requires, this will run the function in separate theads

    :param number_of_threads - how many threads to create
    :param function_to_run - the function to run in those threads
    :param data - the data to run the function with (will be split evenly across the threads)
    :param args - other positional arguments required by the fucntion if any
    """
    data_length = len(data)
    assert number_of_threads > 0 and data_length > 0
    threads = [Thread()] * number_of_threads
    # Each thread will process this much of the data
    number_of_data_slices = round(data_length / number_of_threads)
    thread_count = 0
    # Chunk up the data by number of slices to process in each thread and kick off the threads
    data_chunks = [data[idx:idx+number_of_data_slices] for idx in range(0, data_length, number_of_data_slices)]
    data_chunks_length = len(data_chunks)
    for thread in range(number_of_threads):
        # If there is no data to process then just skip creation of the thread
        if len(data) == 0 or thread + 1 > data_chunks_length:
            continue
        data_to_run_with = data_chunks[thread]
        threads[thread] = Thread(target=function_to_run, args=(data_to_run_with, *args))
        threads[thread].start()
        thread_count += 1
    # Wait for all the threads to finish
    for thread in range(thread_count):
        threads[thread].join()
